#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>

#include "vt.h"
#include "misc.h"
#include "fdset.h"
#include "taztv_xio.h"
#include "vbi.h"
#include "lang.h"
#include "cache.h"
#include "ui.h"
#include "alevt.h"

extern int debug;

static Display *dpy;
static int alevtrunning=0;

int end_alevt = 0;

static pthread_t alevt_thread;

/* current args for next start() */
static char *geometry;

char vbi_name [255] = "";

static struct xio *xio;
static struct vbi *vbi;
static int fine_tune = 1; // auto = 999;
static int erc = 1;
static int newbttv = -1;

void init_alevt(Display *d)
{
  dpy=d;
}

static struct vtwin *
start(int argc, char **argv, struct vtwin *parent, int pgno, int subno)
{
    struct vtwin *win;

    if (vbi_name[0]==0) {
                         strcpy(vbi_name,"/dev/vbi");
    }

    if (vbi == 0)
	vbi = vbi_open(vbi_name, cache_open(), fine_tune, newbttv);

    if (vbi == 0)
       {
	 vbi = vbi_open("/dev/v4l/vbi0", cache_open(), fine_tune, newbttv);
         if (vbi!=0) strcpy(vbi_name,"/dev/v4l/vbi0");
       }
    if (vbi == 0)
       {
	 vbi = vbi_open("/dev/video/vbi0", cache_open(), fine_tune, newbttv);
         if (vbi!=0) strcpy(vbi_name,"/dev/video/vbi0");
       }

    if (vbi == 0) {
	if (debug) fprintf(stderr,"cannot open %s", vbi_name);
        return NULL;
    }

    if (vbi->cache)
	vbi->cache->op->mode(vbi->cache, CACHE_MODE_ERC, erc);

    if (xio == 0)
	xio = xio_open_dpy(dpy, argc, argv);

    if (xio == 0){
        return NULL;
	if (debug) fprintf(stderr, "alevt-cannot init");
    }

    parent = vtwin_new(xio, vbi, geometry, parent, pgno, subno);

    if (parent == 0) {
	if (debug) fprintf(stderr,"alevt-cannot create window");
        return NULL;

    }
    return parent;
}




void
*alevt_main(void *arg)
{
    void *ret = 0;

    alevtrunning=1;
    end_alevt=0;

    xio_event_loop();

    if (end_alevt) xio_close_dpy(xio);
    xio=0;
    vbi_close(vbi);
    vbi=0;
    alevtrunning=0;

    if (end_alevt) { if (debug) fprintf(stderr,"Alevt stop by taztv \n"); }
    else { if (debug) fprintf(stderr,"Alevt stop by alevt \n"); }

    pthread_exit (ret);

}

void start_alevt(void){

    int *arg=0;
    int argc = 1;
    char *name = "taztv-alevt";
    char **argv = &name;

    xio = 0;
    vbi = 0;
    geometry=0;

    if (alevtrunning) return;

    setprgname(argv[0]);

    fdset_init(fds);

    if (start(argc, argv, 0, 0x100, ANY_SUB)==NULL) return;

    pthread_create(&alevt_thread, NULL, alevt_main,(void *) arg);

    if (debug) fprintf(stderr,"Alevt start\n");
}

void stop_alevt(void){
  void *ret;

  if (!alevtrunning) return;
  end_alevt=1;
  pthread_join(alevt_thread, &ret);

}

void toggle_alevt(void){

  if (alevtrunning) stop_alevt(); else  start_alevt();

}

/* name stuff from alevt-date.c */

static char station_name[16];
static char last_name[16];
static int name_not_find;

static void
event(void *_, struct vt_event *ev)
{
      u8 *s = ev->p1;
      u8  *c;
      int i;

    if (!name_not_find) return;

    switch (ev->type)
    {
	/* vbi may generate EV_PAGE, EV_HEADER, EV_XPACKET */
	/* for event arguments see vt.h */

	case EV_HEADER: // a new title line (for running headers)

//	    if (ev->i2 & PG_OUTOFSEQ)
//		break;

            for (i=0;i<9;i++) {
                                c=&s[i+8];
                                if  ((*c<32) || (*c>127))
                                    last_name[i]=32;
                                else
                                    last_name[i]=*c;
                               }
            last_name[9]=0;
            c=&last_name[8];

            while (c>=(u8 *)last_name && *c==32) { *c=0; c--; }

            c=&last_name[0];
            while (*c==32 && *c!=0) {c++; }

            if (last_name[0]!=0) strcpy(station_name,c);

            name_not_find=0;

	    break;
    }
}

char *get_station_name( char **name){

       char *prg_name = "taztv-alevt";
       char **argv = &prg_name;

       int big_buf = -1;
#define TIME_OUT 2
       time_t t;

       sleep(1);

       stop_alevt();
       strcpy(station_name, NO_STATION_NAME);

       setprgname(argv[0]);

       fdset_init(fds);

       vbi = vbi_open(vbi_name, 0, 1, big_buf);	// open device
       if (not vbi) return station_name;

       vbi_add_handler(vbi, event, 0);	// register event handler

       name_not_find=0;
       t=time(NULL);
       while (time(NULL)>t) fdset_select(fds, -1);

       name_not_find=1;
       t=time(NULL);
       while (name_not_find) {
                               fdset_select(fds, -1);		// call scheduler
                               if ((time(NULL)-t)>TIME_OUT) name_not_find =0;
       }

       /*reached */
       vbi_del_handler(vbi, event, 0);
       vbi_close(vbi);
       vbi=0;

       printf ("Station name = %s\n",station_name);

       if (name != NULL) *name = station_name;

       return station_name;
}


