/*************************************************************************
* main_gui.c                                                             *
* Maupatz     2001                                                       *
* taztv is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by   *
* the Free Software Foundation; either version 2 of the License, or      *
* (at your option) any later version.                                    *
*                                                                        *
* Taztv is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
* GNU General Public License for more details.                           *
*                                                                        *
* You should have received a copy of the GNU General Public License      *
* along with this program; if not, write to the Free Software            *
*                                                                        *
*************************************************************************/

#include "taztv.h"
#include "video.h"
#include "alevt.h"
#include "readconfig.h"
#include "main_gui.h"
#include <pthread.h>
#include "gui.h"
#include "gui_list.h"
#include "gui_button.h"
#include "gui_labelbutton.h"
#include "gui_checkbox.h"
#include "gui_label.h"
#include "gui_dnd.h"
#include "gui_slider.h"
#include "gui_parseskin.h"
#include "gui_image.h"
#include "gui_browser.h"
#include "i18n.h"
#include "decode_gui.h"
#include "mixer.h"

static Window          main_win;
static widget_list_t  *main_widget_list;
static ImlibImage     *main_bg_image;

static int datachanged = 0;

static widget_t *ChTvLbl;
static char   ChTvVal[40];
static int    CurTvNum;

static char   *tmpchr;
static char   tmpv[32];

static widget_t  *BriValLbl;
static int BriVal;
static char BricharVal[4];

static widget_t  *ConValLbl;
static int ConVal;
static char ConcharVal[4];

static widget_t  *ColValLbl;
static int ColVal;
static char ColcharVal[4];

static widget_t  *HueValLbl;
static int HueVal;
static char HuecharVal[4];

static widget_t  *SrcValLbl;
static char *SrccharVal;

static widget_t  *NormValLbl;
static int NormVal;
static char *NormcharVal;

static widget_t  *MixerValLbl;
static int MixerVal;
static char MixercharVal[16];


static widget_t  *ChtabValLbl;

static int main_mask_event = ButtonPressMask | ButtonReleaseMask | PointerMotionMask |
	                     ExposureMask;

static int end_main_gui=0;
static int main_gui_running=0;
static pthread_t main_gui_thread;

/* main gui do */

#define NEXTCH     1
#define PREVCH     2

/*
  Change channel
  Mute sound
  Image control : Hue Brigthness Contrast Color
  End taztv
  Set source
  Set Norm
  Decode Menu
  Fullscreen
  Start alevt
*/

#define STEP_IMG 655

static void inc_img_val ( int *val) {
    *val +=STEP_IMG;
    if (*val>65535) *val=65535;
}

static void dec_img_val ( int *val) {
    *val -=STEP_IMG;
    if (*val<0) *val=0;
}

static void convert_img_val (int val, char *s) {
     int v;
    v = val / STEP_IMG;
    if (v<0  ) v=0;
    if (v>100) v=100;
    sprintf (s,"%3d",v);
}

static void convert_mixer_val (int val, char *s) {
     int v;
    v=val&0x7f;
    if (v<0  ) v=0;
    if (v>=100) v=99;
    sprintf (s,"%3d",v);
}

static void main_ch_brightness  (widget_t *w, void *data ) {

   if ( data == (void *) NEXTCH) {
     inc_img_val (&BriVal);
  } else if ( data == (void *) PREVCH) dec_img_val (&BriVal);
  setbrightness(BriVal);
  convert_img_val (BriVal, BricharVal);
  label_change_label (main_widget_list,BriValLbl,BricharVal);
  // printf ("%d %s\n",BriVal,BricharVal);
  datachanged = 1;
}

static void main_ch_contrast (widget_t *w, void *data ) {

  if ( data == (void *) NEXTCH) {
     inc_img_val (&ConVal);
  } else if ( data == (void *) PREVCH) dec_img_val (&ConVal);
  setcontrast(ConVal);
  convert_img_val (ConVal, ConcharVal);
  label_change_label (main_widget_list,ConValLbl,ConcharVal);
  // printf ("%d %s\n",ConVal,ConcharVal);
  datachanged = 1;

}

static void main_ch_color (widget_t *w, void *data ) {
  if ( data == (void *) NEXTCH) {
     inc_img_val (&ColVal);
  } else if ( data == (void *) PREVCH) dec_img_val (&ColVal);
  setcolour(ColVal);
  convert_img_val (ColVal, ColcharVal);
  label_change_label (main_widget_list,ColValLbl,ColcharVal);
  // printf ("%d %s\n",ColVal,ColcharVal);
  datachanged = 1;
}

static void main_ch_hue (widget_t *w, void *data ) {
  if ( data == (void *) NEXTCH) {
     inc_img_val (&HueVal);
  } else if ( data == (void *) PREVCH) dec_img_val (&HueVal);
  sethue(HueVal);
  convert_img_val (HueVal, HuecharVal);
  label_change_label (main_widget_list,HueValLbl,HuecharVal);
  // printf ("%d %s\n",HueVal,HuecharVal);
  datachanged = 1;
}

static void main_ch_channel (widget_t *w, void *data ){
 if ( data == (void *) NEXTCH) {
     inc_tv_channel();
  } else if ( data == (void *) PREVCH)  dec_tv_channel();

}

static void main_ch_source(widget_t *w, void *data ){
  if ( data == (void *) NEXTCH) {
     SrccharVal = inc_source();
  } else if ( data == (void *) PREVCH) SrccharVal = dec_source(); ;
  label_change_label (main_widget_list,SrcValLbl,SrccharVal);
  // printf ("%s\n",SrccharVal);
  datachanged = 1;
}

static void main_ch_norm(widget_t *w, void *data ){
 if ( data == (void *) NEXTCH) {
     NormVal = inc_norm();
  } else if ( data == (void *) PREVCH) NormVal = dec_norm();
  NormcharVal=get_norm_name(NormVal);
  label_change_label (main_widget_list,NormValLbl,NormcharVal);
  // printf ("%s\n",NormcharVal);
  datachanged = 1;
}

static void main_gui_alevt (widget_t *w, void *data ){
     toggle_alevt();
}

static void main_gui_mutetv (widget_t *w, void *data ){
  SEND_KEVENT(XK_M);
}


static void main_ch_channeltab (widget_t *w, void *data ){
  if ( data == (void *) NEXTCH) {
     tmpchr = inc_freqtab();
  } else if ( data == (void *) PREVCH) tmpchr = dec_freqtab();
  label_change_label (main_widget_list,ChtabValLbl,tmpchr);
  // printf ("%s\n",tmpchr);
  // getfreqtab(&tmpchr);
  // printf ("%s\n",tmpchr);
  datachanged = 1;
}

static  void main_gui_fs(widget_t *w, void *data ) {
   SEND_KEVENT(XK_F);
 }

static void main_ch_vol(widget_t *w, void *data )
{
  if ( data == (void *) NEXTCH)
  {
    SEND_KEVENT(XK_Up);
  } else if ( data == (void *) PREVCH) SEND_KEVENT(XK_Down);
  datachanged = 1;
}

static void main_ch_format(widget_t *w, void *data )
{
 SEND_KEVENT(XK_C);
}

static  void main_gui_decode(widget_t *w, void *data ) {
    toggle_decode_gui();
 }


 /*
 * Leaving main_gui
 */

static void main_gui_exit(widget_t *w, void *data) {

  main_gui_running=0;

   abort_decode_gui();

  XUnmapWindow(gDisplay, main_win);

  gui_list_free(main_widget_list->l);
  free(main_widget_list);
  main_widget_list = NULL;

  XDestroyWindow(gDisplay, main_win);
  main_win = 0;

  if (debug) fprintf(stderr,"main gui stop\n");

  pthread_exit((void *) 0 );

}

static void main_gui_close(widget_t *w, void *data ) {
  end_main_gui = 1;
}

static void main_taztv_exit(widget_t *w, void *data) {
     not_end_taztv = 0;
     end_main_gui = 1;
}

static void *main_gui_loop (void *arg){

   XEvent event[1];
   widget_t *w=NULL;
   void *data=NULL;
   int x=0,y=0,ox=0,oy=0;
   int move_win=0;
   Window wr;
   int k;

 while (!end_main_gui) {

       usleep(10000);
      /* Thread Method is wonderful */
       tmpchr = get_current_tv_channel(&k);
       if (k!=CurTvNum) {
           CurTvNum=k;
           sprintf (ChTvVal," %d:%s",CurTvNum,tmpchr);
           label_change_label (main_widget_list,ChTvLbl,ChTvVal);
           // printf ("%s\n",ChTvVal);
       }

       get_mixer_input_level(&k);
       if (k!=MixerVal)
       {
         MixerVal=k;
         convert_mixer_val (MixerVal,MixercharVal);
         label_change_label (main_widget_list,MixerValLbl,MixercharVal);
       }

       if (XPending(gDisplay)) {

       while (XCheckWindowEvent (gDisplay,main_win,main_mask_event,event)!=False){
             switch(event->type) {
              case Expose:
               paint_widget_list (main_widget_list);
              break;

              case MotionNotify:
                 if (move_win) {
                                  XLOCK();
                                  XTranslateCoordinates(gDisplay,main_win,
                                  RootWindow(gDisplay,DefaultScreen(gDisplay)),
                                  0,0,&x,&y,&wr);
                                  x = x + event->xmotion.x - ox;
                                  y = y + event->xmotion.y - oy;
                                  XMoveWindow(gDisplay, main_win, x, y);
                                  XSync(gDisplay, False);
                                  XSync(gDisplay, False);
  /* let's go faster :-) */
 while (XCheckWindowEvent (gDisplay,main_win,PointerMotionMask,event)==True);
                                  XUNLOCK();

                           /* fin */
    //                   printf ("%d %d %d %d\n",event->xmotion.x_root, event->xmotion.y_root,
    //                     event->xmotion.x, event->xmotion.y );
                 } else
                 motion_notify_widget_list (main_widget_list,
	  			 event->xbutton.x, event->xbutton.y);
              break;

              case ButtonPress:
                move_win =
                !click_notify_widget_list (main_widget_list, event->xbutton.x,
			        event->xbutton.y, 0);
                 if (move_win) { ox = event->xbutton.x; oy =  event->xbutton.y; }
              break;

              case ButtonRelease:
                  move_win = 0;
                  click_notify_widget_list (main_widget_list, event->xbutton.x,
				event->xbutton.y, 1);

              break;
          /*
              case ConfigureNotify:
                Imlib_apply_image(gImlib_data, main_bg_image,main_win );
                paint_widget_list (main_widget_list);
                XSync(gDisplay, False);
              break;
           */
            }
       }
//          k=XPending(gDisplay);
//          if (k) { printf ("Reste %d\n",k); } else  printf ("Reste rien\n");
   }
 }

 /* Data to config file */
 if (datachanged) {
     open_config_file();

     sprintf (tmpv,"%d",BriVal);
     save_config_value("brightness",tmpv,AT_BEGIN);

     sprintf (tmpv,"%d",ConVal);
     save_config_value("contrast",tmpv,AT_BEGIN);

     sprintf (tmpv,"%d",ColVal);
     save_config_value("colour",tmpv,AT_BEGIN);

     sprintf (tmpv,"%d",HueVal);
     save_config_value("hue",tmpv,AT_BEGIN);

     getfreqtab(&tmpchr);
     save_config_value("freqtab",tmpchr,AT_BEGIN );

     save_config_value("norm",get_norm_name2(NormVal),AT_BEGIN );

     save_config_value("source",SrccharVal,AT_BEGIN );

     close_config_file();
 }

 main_gui_exit (w,data);
 return 0;
}

int start_main_gui(void){

  int *arg=0;
  GC                      gc;
  XSizeHints              hint;
  XWMHints               *wm_hint;
  XSetWindowAttributes    attr;
  int                     screen;
  char                    title[] = {"Taztv Main gui window"};
  char                    *main_gui_title;
  Atom                    prop;
  MWMHints                mwmhints;
  XClassHint             *xclasshint;

  if (main_gui_running) return -1;

  datachanged = 0;

  main_gui_title  = _("  Main Taztv Command  ");

  if (!(main_bg_image = Imlib_load_image(gImlib_data,
				       gui_get_skinfile("MainBG")))) {
    if (debug) fprintf(stderr, "Taztv main gui : couldn't find image for background\n");
    return -1;
  }

  screen = DefaultScreen(gDisplay);
  hint.x = 0;
  hint.y = 0;
  hint.width = main_bg_image->rgb_width;
  hint.height = main_bg_image->rgb_height;
  hint.flags = PSize; // PPosition | PSize;


  attr.background_pixmap=None;
  attr.background_pixel=0;
  attr.border_pixmap=CopyFromParent;
  attr.border_pixel=0;
  attr.bit_gravity=ForgetGravity;
  attr.win_gravity=NorthWestGravity;
  attr.backing_store=0;
  attr.backing_planes=0;
  attr.backing_pixel=0;
  attr.save_under=False;
  attr.event_mask=0;
  attr.do_not_propagate_mask=0;
  attr.override_redirect=True;
  attr.colormap=CopyFromParent;
  attr.cursor=None;

  main_win = XCreateWindow (gDisplay, DefaultRootWindow(gDisplay),
			  hint.x, hint.y, hint.width, hint.height, 0,
			  CopyFromParent, CopyFromParent,
			  CopyFromParent,
			  0, &attr);

  XSetStandardProperties(gDisplay, main_win, title, title,
			 None, NULL, 0, &hint);

  /*
   * wm, no border please
   */

  prop = XInternAtom(gDisplay, "_MOTIF_WM_HINTS", True);
  mwmhints.flags = MWM_HINTS_DECORATIONS;
  mwmhints.decorations = 0;

  XChangeProperty(gDisplay, main_win, prop, prop, 32,
                  PropModeReplace, (unsigned char *) &mwmhints,
                  PROP_MWM_HINTS_ELEMENTS);

  XSetTransientForHint(gDisplay, main_win, None);

/* set xclass */
  if((xclasshint = XAllocClassHint()) != NULL) {
    xclasshint->res_name = "Taztv Panel";
    xclasshint->res_class = "Taztv";
    XSetClassHint(gDisplay, main_win, xclasshint);
    XFree(xclasshint);
  }

  gc = XCreateGC(gDisplay, main_win, 0, 0);
  XSetGraphicsExposures(gDisplay,gc,False);

  XSelectInput(gDisplay, main_win, main_mask_event);

  wm_hint = XAllocWMHints();
  if (wm_hint != NULL) {
    wm_hint->input = True;
    wm_hint->initial_state = NormalState;
    wm_hint->flags = InputHint | StateHint;
    XSetWMHints(gDisplay, main_win, wm_hint);
    XFree(wm_hint);
  }

  XLOCK();
  Imlib_apply_image(gImlib_data, main_bg_image, main_win);

  XSync(gDisplay, False);
  XSync(gDisplay, False);

  /*
   * Widget-list
   */
  main_widget_list = (widget_list_t *) malloc (sizeof (widget_list_t));
  main_widget_list->l = gui_list_new ();
  main_widget_list->focusedWidget = NULL;
  main_widget_list->pressedWidget = NULL;
  main_widget_list->win           = main_win;
  main_widget_list->gc            = gc;

/**************** Init Value *******************************/

   /*
  Change channel
  Mute sound
  Image control : Hue Brigthness Contrast Color
  End taztv
  Set source
  Set Norm
  Scan TV
  Decode Menu
  Start alevt
*/

  /* image */
  gui_list_append_content (main_widget_list->l,
			   create_image (gui_get_skinX("tazmain"),
					 gui_get_skinY("tazmain"),
					 gui_get_skinfile("tazmain")));

  /* Close window */
  gui_list_append_content (main_widget_list->l,
			   create_label_button (gui_get_skinX("MainDismiss"),
						gui_get_skinY("MainDismiss"),
						CLICK_BUTTON, _("Dismiss"),
						main_gui_close, NULL,
						gui_get_skinfile("MainDismiss"),
						gui_get_ncolor("MainDismiss"),
						gui_get_fcolor("MainDismiss"),
						gui_get_ccolor("MainDismiss")));
  /* End taztv */
  gui_list_append_content (main_widget_list->l,
			   create_label_button (gui_get_skinX("MainEnd"),
						gui_get_skinY("MainEnd"),
						CLICK_BUTTON, _("Quit"),
						main_taztv_exit, NULL,
						gui_get_skinfile("MainEnd"),
						gui_get_ncolor("MainEnd"),
						gui_get_fcolor("MainEnd"),
						gui_get_ccolor("MainEnd")));
   /* main label */
   gui_list_append_content (main_widget_list->l,
			   create_label (gui_get_skinX("MainTvLbl"),
					 gui_get_skinY("MainTvLbl"),
					 (int) strlen(main_gui_title),
                                          main_gui_title,
					 gui_get_skinfile("MainTvLbl")));


  /* Change channel label (32 + 5 characters) + arrowup + arrowdown */

   tmpchr = get_current_tv_channel(&CurTvNum);
   sprintf (ChTvVal," %d:%s",CurTvNum,tmpchr);

   gui_list_append_content (main_widget_list->l,
			   ChTvLbl = create_label (gui_get_skinX("MainChTvLbl"),
					 gui_get_skinY("MainChTvLbl"),
					 16,
                                         ChTvVal,
					 gui_get_skinfile("MainChTvLbl")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainChNext"),
				          gui_get_skinY("MainChNext"),
					  main_ch_channel,(void*)NEXTCH ,
					  gui_get_skinfile("MainChNext")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainChPrev"),
					  gui_get_skinY("MainChPrev"),
					  main_ch_channel, (void*)PREVCH,
					  gui_get_skinfile("MainChPrev")));


 /* Brightness  */
   getbrigthness(&BriVal);
   convert_img_val (BriVal,BricharVal);
   gui_list_append_content (main_widget_list->l,
			   create_label (gui_get_skinX("MainBriLbl"),
					 gui_get_skinY("MainBriLbl"),
					 3,
                                         _("Bri"),
					 gui_get_skinfile("MainBriLbl")));

   gui_list_append_content (main_widget_list->l,
			   BriValLbl = create_label (gui_get_skinX("MainBriVal"),
					 gui_get_skinY("MainBriVal"),
					 3,
                                         BricharVal,
					 gui_get_skinfile("MainBriVal")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainBriNext"),
				          gui_get_skinY("MainBriNext"),
					  main_ch_brightness,(void*)NEXTCH ,
					  gui_get_skinfile("MainBriNext")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainBriPrev"),
					  gui_get_skinY("MainBriPrev"),
					  main_ch_brightness, (void*)PREVCH,
					  gui_get_skinfile("MainBriPrev")));
 /* Constrast   */

   getcontrast(&ConVal);
   convert_img_val (ConVal,ConcharVal);
   gui_list_append_content (main_widget_list->l,
			   create_label (gui_get_skinX("MainConLbl"),
					 gui_get_skinY("MainConLbl"),
					 3,
                                         _("Con"),
					 gui_get_skinfile("MainConLbl")));

    gui_list_append_content (main_widget_list->l,
			   ConValLbl = create_label (gui_get_skinX("MainConVal"),
					 gui_get_skinY("MainConVal"),
					 3,
                                          ConcharVal,
					 gui_get_skinfile("MainConVal")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainConNext"),
				          gui_get_skinY("MainConNext"),
					  main_ch_contrast,(void*)NEXTCH ,
					  gui_get_skinfile("MainConNext")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainConPrev"),
					  gui_get_skinY("MainConPrev"),
					  main_ch_contrast, (void*)PREVCH,
					  gui_get_skinfile("MainConPrev")));
 /* Color       */
   getcolour(&ColVal);
   convert_img_val (ColVal,ColcharVal);
   gui_list_append_content (main_widget_list->l,
			                 create_label (gui_get_skinX("MainColLbl"),
					 gui_get_skinY("MainColLbl"),
					 3,
                                         _("Col"),
					 gui_get_skinfile("MainColLbl")));

   gui_list_append_content (main_widget_list->l,
			   ColValLbl =  create_label (gui_get_skinX("MainColVal"),
					 gui_get_skinY("MainColVal"),
					 3,
                                          ColcharVal,
					 gui_get_skinfile("MainColVal")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainColNext"),
				          gui_get_skinY("MainColNext"),
					  main_ch_color,(void*)NEXTCH ,
					  gui_get_skinfile("MainColNext")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainColPrev"),
					  gui_get_skinY("MainColPrev"),
					  main_ch_color, (void*)PREVCH,
					  gui_get_skinfile("MainColPrev")));
 /* Hue         */
   gethue(&HueVal);
   convert_img_val (HueVal,HuecharVal);
   gui_list_append_content (main_widget_list->l,
			    create_label (gui_get_skinX("MainHueLbl"),
					 gui_get_skinY("MainHueLbl"),
					 3,
                                         _("Hue"),
					 gui_get_skinfile("MainHueLbl")));

    gui_list_append_content (main_widget_list->l,
			  HueValLbl = create_label (gui_get_skinX("MainHueVal"),
					 gui_get_skinY("MainHueVal"),
					 3,
                                         HuecharVal,
					 gui_get_skinfile("MainHueVal")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainHueNext"),
				          gui_get_skinY("MainHueNext"),
					  main_ch_hue,(void*)NEXTCH ,
					  gui_get_skinfile("MainHueNext")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainHuePrev"),
					  gui_get_skinY("MainHuePrev"),
					  main_ch_hue, (void*)PREVCH,
					  gui_get_skinfile("MainHuePrev")));
 /* Source      */

   getsource(&SrccharVal);
   gui_list_append_content (main_widget_list->l,
			  SrcValLbl = create_label (gui_get_skinX("MainSrcLbl"),
					 gui_get_skinY("MainSrcLbl"),
					 16,
                                         SrccharVal,
					 gui_get_skinfile("MainSrcLbl")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainSrcNext"),
				          gui_get_skinY("MainSrcNext"),
					  main_ch_source,(void*)NEXTCH ,
					  gui_get_skinfile("MainSrcNext")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainSrcPrev"),
					  gui_get_skinY("MainSrcPrev"),
					  main_ch_source, (void*)PREVCH,
					  gui_get_skinfile("MainSrcPrev")));
 /* Norme       */

   getnorm(&NormVal);
   NormcharVal=get_norm_name(NormVal);
   gui_list_append_content (main_widget_list->l,
			NormValLbl=create_label (gui_get_skinX("MainNormLbl"),
					 gui_get_skinY("MainNormLbl"),
					 7,
                                         NormcharVal,
					 gui_get_skinfile("MainNormLbl")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainNormNext"),
				          gui_get_skinY("MainNormNext"),
					  main_ch_norm,(void*)NEXTCH ,
					  gui_get_skinfile("MainNormNext")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainNormPrev"),
					  gui_get_skinY("MainNormPrev"),
					  main_ch_norm, (void*)PREVCH,
                                          gui_get_skinfile("MainNormPrev")));

 /* Mute     */

 gui_list_append_content (main_widget_list->l,
			   create_label_button (gui_get_skinX("MainMutetv"),
						gui_get_skinY("MainMutetv"),
						CLICK_BUTTON, _("Mute"),
						main_gui_mutetv, NULL,
						gui_get_skinfile("MainMutetv"),
						gui_get_ncolor("MainMutetv"),
						gui_get_fcolor("MainMutetv"),
						gui_get_ccolor("MainMutetv")));

 /* alevt  */
  gui_list_append_content (main_widget_list->l,
			   create_label_button (gui_get_skinX("MainAlevt"),
						gui_get_skinY("MainAlevt"),
						CLICK_BUTTON, _("AleVt"),
						main_gui_alevt, NULL,
						gui_get_skinfile("MainAlevt"),
						gui_get_ncolor("MainAlevt"),
						gui_get_fcolor("MainAlevt"),
						gui_get_ccolor("MainAlevt")));

/* Scantv      */

 gui_list_append_content (main_widget_list->l,
			   create_label_button (gui_get_skinX("MainFs"),
						gui_get_skinY("MainFs"),
						CLICK_BUTTON, _("FullScreen"),
						main_gui_fs, NULL,
						gui_get_skinfile("MainFs"),
						gui_get_ncolor("MainFs"),
						gui_get_fcolor("MainFs"),
						gui_get_ccolor("MainFs")));

 /* Channel tab */

   getfreqtab(&tmpchr);
   gui_list_append_content (main_widget_list->l,
			   ChtabValLbl = create_label (gui_get_skinX("MainChTabLbl"),
					 gui_get_skinY("MainChTabLbl"),
					 16,
                                         tmpchr,
					 gui_get_skinfile("MainChTabLbl")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainChTabNext"),
				          gui_get_skinY("MainChTabNext"),
					  main_ch_channeltab,(void*)NEXTCH ,
					  gui_get_skinfile("MainChTabNext")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainChTabPrev"),
					  gui_get_skinY("MainChTabPrev"),
					  main_ch_channeltab, (void*)PREVCH,
					  gui_get_skinfile("MainChTabPrev")));

 /*  Volume */
   get_mixer_input_level(&MixerVal);
   convert_mixer_val (MixerVal,MixercharVal);
   gui_list_append_content (main_widget_list->l,
			    create_label (gui_get_skinX("MainMixerLbl"),
					 gui_get_skinY("MainMixerLbl"),
					 3,
                                         _("Vol"),
					 gui_get_skinfile("MainMixerLbl")));

    gui_list_append_content (main_widget_list->l,
			  MixerValLbl = create_label (gui_get_skinX("MainMixerVal"),
					 gui_get_skinY("MainMixerVal"),
					 3,
                                         MixercharVal,
					 gui_get_skinfile("MainMixerVal")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainMixerNext"),
				          gui_get_skinY("MainMixerNext"),
					  main_ch_vol,(void*)NEXTCH ,
					  gui_get_skinfile("MainMixerNext")));

    gui_list_append_content (main_widget_list->l,
			   create_button (gui_get_skinX("MainMixerPrev"),
					  gui_get_skinY("MainMixerPrev"),
					  main_ch_vol, (void*)PREVCH,
					  gui_get_skinfile("MainMixerPrev")));

 /* format */
    gui_list_append_content (main_widget_list->l,
			   create_label_button (gui_get_skinX("MainFormat"),
						gui_get_skinY("MainFormat"),
						CLICK_BUTTON, _("Format"),
						main_ch_format, NULL,
						gui_get_skinfile("MainFormat"),
						gui_get_ncolor("MainFormat"),
						gui_get_fcolor("MainFormat"),
						gui_get_ccolor("MainFormat")));


 /*  decodage  :  en dernier */

   gui_list_append_content (main_widget_list->l,
			   create_label_button (gui_get_skinX("MainDecode"),
						gui_get_skinY("MainDecode"),
						CLICK_BUTTON, _("Decoding"),
						main_gui_decode, NULL,
						gui_get_skinfile("MainDecode"),
						gui_get_ncolor("MainDecode"),
						gui_get_fcolor("MainDecode"),
						gui_get_ccolor("MainDecode")));

 
  XMapRaised(gDisplay, main_win);

  XSync(gDisplay, False);
  XSync(gDisplay, False);
  XUNLOCK();

  main_gui_running=1;
  end_main_gui=0;
  pthread_create(&main_gui_thread, NULL, main_gui_loop ,(void *) arg);
  if (debug) fprintf (stderr,"Main gui start \n");
  return 0;
}

void abort_main_gui(void){
  void *ret;
  if (main_gui_running==0) return;
  end_main_gui = 1;
  pthread_join(main_gui_thread, &ret);
  return;
}

void toggle_main_gui(void) {

  if (main_gui_running) abort_main_gui(); else  start_main_gui();

}

