/*************************************************************************
* readconfig.c                                                           *
* Maupatz  2001                                                          *
* taztv is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by   *
* the Free Software Foundation; either version 2 of the License, or      *
* (at your option) any later version.                                    *
*                                                                        *
* Taztv is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
* GNU General Public License for more details.                           *
*                                                                        *
* You should have received a copy of the GNU General Public License      *
* along with this program; if not, write to the Free Software            *
*                                                                        *
*************************************************************************/

#include  "taztv.h"
#include  "readconfig.h"
#include  "nagra.h"
#include  "yuyv2rgb.h"
#include  "video.h"
#include  "x11.h"
#include  "frequencies.h"
#include  "nagraudio.h"
#include  "mixer.h"
#include  "gui.h"
#include  "alevt.h"
#include  "kizkool.h"

typedef struct line_list_s {
        char s[100];
        struct line_list_s *next;
} line_list_t;


static char filename[100], line[100], tag[32], val[32], *h;
static FILE *fp;
static line_list_t *line_list = NULL;
static int config_modif=0;

static void free_line_list (line_list_t *l){
    line_list_t *lp;

   while (l!=NULL) {
                     lp = l;
                     l=l->next;
                     free(lp);
   }
}

int  open_config_file(void){

  line_list_t *la=NULL;
  line_list_t *lp=NULL;

  if (line_list!=NULL) return -1;

  sprintf(filename,"%s/%s",user_dir,"taztvrc");
  fp = fopen(filename,"r");

  if (NULL == fp) {

               fp = fopen(filename,"w");
               if (NULL == fp)  return -1;
               fputs("# Taztv config file\n",fp);
               fclose(fp);
  }

  fp = fopen(filename,"r");
  if (NULL == fp) return -1 ;

  while (fgets(line,99,fp)!=NULL) {
       lp = malloc (sizeof(line_list_t));
       if (lp==NULL) { free_line_list (line_list); line_list=NULL; return -1; }
       lp->next = NULL;
       if (line_list==NULL) line_list=la=lp; else { la->next = lp; la=lp; }
       strcpy (la->s,line);
  }

  fclose (fp);
  fp=NULL;
  config_modif=0;
  return  0;
}

int  close_config_file(void){

  line_list_t *l;

 if (line_list==NULL) return -1;
 if (config_modif)
 {
  fp = fopen(filename,"w");
  if (NULL == fp)  return -1;

  l = line_list;
  do {
      fputs (l->s,fp);
      l = l->next;
  } while (l!= NULL);
  fclose (fp);
  fp = NULL;
 }
 free_line_list (line_list);
 line_list = NULL;
 config_modif=0;
 return 0;
}


static line_list_t *find_tag (char *lbl)
 {
   line_list_t *l;
   int i;

      if (line_list==NULL) return NULL;
      l = line_list;

     /* try to find value */
      while (l!=NULL) {

         strcpy (line,l->s);

           /*** suppress espace before ***/
        for (i=0;i<strlen(line); i++) if (line[i]!=' ') break;
        if (i!=0){
                   h = line + i; i=0;
                   while (*h!=0){*(line+i)=*h ; h++; i++; }
                   *(line+i)=0;
             }

        if (line[0] == '\n' || line[0] == '#' || line[0] == '%')
        {
          l=l->next;
          continue;
        }

        if (2 != sscanf(line,"%31[^= ] = %31[^\n]",tag,val)) {
          l=l->next;
          continue;
        }

        /*** supress espace after ***/
        for (h = val+strlen(val) - 1; h > val && *h == ' '; h--) *h='\0';

        if  (strcmp(tag,lbl)==0) break;

       l=l->next;
     }

   return l;
 }

int  save_config_value(char *lbl, char *lblval, int pref) {
     line_list_t *l;
     line_list_t *lp;

     l =  find_tag (lbl);

     /* not found : create a new value */
      if (l == NULL) {
                      l = malloc( sizeof(line_list_t));
                      if (l==NULL) return -1;
                      sprintf (l->s,"  %s = %s \n", lbl, lblval);
                      switch (pref) {
                                  case  AT_BEGIN:
                                     l->next = line_list;
                                     line_list = l;
                                  break;

                                  case  AT_END:
                                       lp = line_list;
                                       while (lp->next != NULL) lp = lp->next;
                                       lp->next = l; l->next = NULL;
                                  break;
                      }

      } else
            { /* Found store new value */
              sprintf (l->s,"  %s = %s \n", lbl, lblval);
            }
 config_modif=1;
 return 0;
}

int  get_config_value(char *lbl, char *lblval) {

     line_list_t *l;
     l =  find_tag (lbl);
     if (l==NULL) return -1;

     strcpy(lblval,val);

     return 0;
}

int load_last_channel(){
  int c;
  sprintf(filename,"%s/%s",user_dir,"lastchannel");
  fp = fopen(filename,"r");
  if (NULL == fp) {
    if (debug) fprintf(stderr,"Can't open channel file %s, using defaults\n",filename);
    return 1;
  }
  fscanf (fp,"%d",&c);
  fclose (fp);
  fp = NULL;
  return c;
}

int save_last_channel(int *c){
  sprintf(filename,"%s/%s",user_dir,"lastchannel");
  fp = fopen(filename,"w");
  if (NULL == fp) {
    if (debug) fprintf(stderr,"Can't save channel file %s",filename);
    return -1;
  }
  fprintf(fp,"%d",*c);
  fclose (fp);
  fp=NULL;
  return *c;

}

void parse_config()
{

  int i,nl = 0;
   /***** Init ******/
    tv_channel_list[0].norm=-1;
    tv_channel_list[0].freq=0;
    tv_channel_list[0].name[0]=0;
    tv_channel_list[0].channel[0]=0;

 /****************** Open file ***************************/
  sprintf(filename,"%s/%s",user_dir,"taztvrc");
  fp = fopen(filename,"r");
  if (NULL == fp) {
    if (debug) fprintf(stderr,"Can't open config file %s, using defaults\n",filename);
    return;
  }

 /************************Parse File **********************/

  while (NULL != fgets(line,99,fp)) {
    nl++;

       /*** suppress espace before ***/
    for (i=0;i<strlen(line); i++) if (line[i]!=' ') break;
    if (i!=0){
    h = line + i; i=0;
    while (*h!=0){*(line+i)=*h ; h++; i++; }
    *(line+i)=0;
             }

    if (line[0] == '\n' || line[0] == '#' || line[0] == '%')
      continue;

    if (2 != sscanf(line,"%31[^= ] = %31[^\n]",tag,val)) {
      if (debug) fprintf(stderr,"%s: line %d: parse error\n",filename,nl);
      continue;
    }

        /*** supress espace after ***/

   for (h = val+strlen(val) - 1; h > val && *h == ' '; h--) *h='\0';


 //  printf(" *%s* *%s* %d %d \n",tag,val, strlen (tag), strlen (val));

/******************************  skin options *******************************/
  if  (strcmp(tag,"skin")==0) {
                                set_skin_name (val);
                                continue;
                              }

/**************************** Parse image options **************************/
  /*
   if  (strcmp(tag,"video_device")==0){
       if (video_device[0]!=0) strcpy(video_device,val);
	               continue;
   }

   if  (strcmp(tag,"vbi_name")==0){
       if (vbi_name[0]!=0) strcpy(vbi_name,val);
	               continue;
   }
 */
   if  (strcmp(tag,"freqtab")==0) {
           i=0;
           while (chanlist_names[i].str!=NULL)
                  if (strcmp(chanlist_names[i].str,val)==0) break; else i++;
           if (chanlist_names[i].str!=NULL) {
             chantab   = i;
             chanlist  = chanlists[chantab].list;
             chancount =  chanlists[chantab].count;
                                            }
           else if (debug) fprintf(stderr,"%s:%d: wrong value for %s\n",filename,nl,tag);
                                continue;
                                   }

   if  (strcmp(tag,"norm")==0) {
         i=1;
         if (strcmp(val,"secam")==0) { i=0; setnorm(VIDEO_MODE_SECAM); }
         if (strcmp(val,"pal")==0)  { i=0; setnorm(VIDEO_MODE_PAL);    }
         if (strcmp(val,"ntsc")==0) { i=0; setnorm(VIDEO_MODE_NTSC);   }
         if (strcmp(val,"auto")==0) { i=0; setnorm(VIDEO_MODE_AUTO);   }
	 if (i && debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
                               continue;
                               }

   if  (strcmp(tag,"source")==0) {
       if (setsource(val)==-1)
       if (debug){
                   fprintf(stderr,"%s:line %d: wrong value for %s\n It Must be : ",filename,nl,tag);
                   for (i=0; video_channel_list[i][0]!=0;i++)
                          fprintf(stderr,"%s ",video_channel_list[i]);
                   fprintf(stderr,"\n");
                 }
	                   continue;
                                 }

   if  (strcmp(tag,"colour")==0) {
        i=atoi (val);
        if ( (i>=0) && (i<=65535) ){
                                    setcolour(i);
        } else
	if (debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	                    continue;
                                }

   if  (strcmp(tag,"brightness")==0) {
          i=atoi (val);
          if ( (i>=0) && (i<=65535) ){
                                    setbrightness(i);
          } else
	if (debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	                         continue;
                                     }


   if  (strcmp(tag,"contrast")==0) {
           i=atoi (val);
          if ( (i>=0) && (i<=65535) ){
                                    setcontrast(i);
          } else
	if (debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	                         continue;
                                    }

   if  (strcmp(tag,"hue")==0) {
            i=atoi (val);
          if ( (i>=0) && (i<=65535) ){
                                    sethue(i);
          } else

	if (debug)  fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	                       continue;
                                     }

 /******************************** Parse image decoder options ***********************/

   if  (strcmp(tag,"decoder")==0) {
         i=1;
         if (strcmp(val,"off")==0)   { i=0; set_decoder_mode (OFF);  }
         if (strcmp(val,"on")==0)    { i=0; set_decoder_mode (ON);   }
         if (strcmp(val,"auto")==0)  { i=0; set_decoder_mode (AUTO); }
	 if (i && debug)  fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	                       continue;
                                  }

   if  (strcmp(tag,"decoder_in_use")==0){
         i = atoi (val);
         switch (i) {
              case 0: case 1:
                set_plug_decoder(i);
              break;
              default:
	         if (debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	      break;
                    }
         continue;
                                        }

    if  (strcmp(tag,"synchro_sound")==0){
         i = atoi (val);
         switch (i) {
              case 0: case 1:
                set_synchro_decoder(i);
              break;
              default:
	         if (debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	      break;
                    }
         continue;
                                        }

   if  (strcmp(tag,"nbcouples")==0){
         i = atoi (val);
         switch (i) {
   case 8: case 12: case 16: case 20:
   case 24: case 28: case 32: case 48:
   case 56: case 64: case 72: case 80:
           set_couples(i);
         break;
         default:
	  if (debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	 break;
                    }
                       continue;
                                  }

  if  (strcmp(tag,"quality")==0){
        i=1;
        if (strcmp(val,"high")==0) { i=0; set_quality_mode(ON);   }
        if (strcmp(val,"low")==0)  { i=0; set_quality_mode(OFF);  }
	if (i && debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	               continue;
                                }

  if  (strcmp(tag,"filter")==0){
        i=1;
        if (strcmp(val,"on")==0)   { i=0; set_filter_mode(ON);}
        if (strcmp(val,"off")==0)  { i=0; set_filter_mode(OFF);}
        if (i && debug)
            fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
                         continue;
                               }

   if (0 == strcmp(tag,"u_filter_threshold")) {
       setfilterparam( KIZKOOL_UTHRESHOLD,atoi (val));
       if (debug)
            fprintf(stderr,"Set u_filter_threshold to : %s \n",val);
                      continue;
   }

   if (0 == strcmp(tag,"v_filter_threshold")) {
       setfilterparam( KIZKOOL_VTHRESHOLD,atoi (val));
       if (debug)
            fprintf(stderr,"Set v_filter_threshold to : %s \n",val);
                      continue;
   }
   if (0 == strcmp(tag,"yuv_filter_threshold")) {
       setfilterparam( KIZKOOL_YUVTHRESHOLD,atoi (val));
       if (debug)
            fprintf(stderr,"Set yuv_filter_threshold to : %s \n",val);
                      continue;
   }
   if (0 == strcmp(tag,"pix_filter_threshold")) {
       setfilterparam( KIZKOOL_PIXFILTERTHRESHOLD,atoi (val));
       if (debug)
            fprintf(stderr,"Set pix_filter_threshold to : %s \n",val);
                      continue;
   }

   if  (strcmp(tag,"format")==0){
        int x ; int y;
        i=sscanf(val,"%d/%d",&x,&y);
        if (i==2) { i=0; set_win_format_ratio(x,y); }
	if (i && debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	               continue;
                                }
				
   if  (strcmp(tag,"fullscreen")==0){
       int x ; int y;
       i=sscanf(val,"%dx%d",&x,&y);
       if (i==2) { i=0; set_fullscreen (x,y); }
       if (i && debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	               continue;
                                }

/**************************** Parse sound decoder options ******************************/
   if  (strcmp(tag,"mixer_device")==0){
       set_mixer_device(val);
       if (debug)
            fprintf(stderr,"Set mixer device to : %s \n",val);
	               continue;
   }
   
   if  (strcmp(tag,"pcm_device")==0){
        set_nagraudio_pcm_device(val);
        if (debug)
            fprintf(stderr,"Set pcm device to : %s \n",val);
	               continue;
   }

   if (0 == strcmp(tag,"recording_level")) {
       set_recording_level(atoi(val));
       if (debug)
            fprintf(stderr,"Set recording level to : %s \n",val);
                      continue;
   }

   if  (strcmp(tag,"sound_input")==0) {
       set_mixer_input_name (val);
       if (debug)
            fprintf(stderr,"Set Mixer input to : %s \n",val);
       continue;
   }

   if  (strcmp(tag,"decode_sound")==0) {
         i=1;
         if (strcmp(val,"off")==0)   { i=0; set_sound_decoder_mode(OFF);  }
         if (strcmp(val,"on")==0)    { i=0;set_sound_decoder_mode(ON);   }
         if (strcmp(val,"auto")==0) { i=0;set_sound_decoder_mode(AUTO); }
	 if (i && debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);

	                       continue;
                                       }

   if (0 == strcmp(tag,"8_bits")) {
   if (set_nagraudio_bit_mode(atoi(val))!=0)
   if (debug) fprintf(stderr,"%s:%d: wrong valor in %s\n",filename,nl,tag);
                               continue;
                                      }

  if (0 == strcmp(tag,"pre_filter")) {
   if (set_nagraudio_pre_filter(atoi(val))!=0)
   if (debug) fprintf(stderr,"%s:%d: wrong valor in %s\n",filename,nl,tag);
                               continue;
                                      }

  if (0 == strcmp(tag,"post_filter")){
     if (set_nagraudio_post_filter(atoi(val))!=0)
     if (debug) fprintf(stderr,"%s:%d: wrong valor in %s\n",filename,nl,tag);
                               continue;
                                      }

  if (0 == strcmp(tag,"output_gain")) {
        if (set_nagraudio_gain(atoi(val))!=0)
	if (debug) fprintf(stderr,"%s:%d: wrong valor in %s\n",filename,nl,tag);
                             continue;
                                      }


/***************************** Parse TV **********************************/

  if  (strcmp(tag,"tv_name")==0){
       /* Ranger le prcdent si il y a tv_freq ou tv_channel */
      if ((tv_channel_list[nbchaines].freq!=0) || ( tv_channel_list[nbchaines].channel[0]!=0)){
       if (nbchaines>=(NB_TV_CHANNEL_MAX - 1)){
        if (debug)
        fprintf(stderr,"Channel limit to %d :line %d  %s : file %s\n",NB_TV_CHANNEL_MAX,nl,tag,filename);
                                              }
        else {
           nbchaines++;
           tv_channel_list[nbchaines].norm=-1;
           tv_channel_list[nbchaines].freq=0;
           tv_channel_list[nbchaines].name[0]=0;
           tv_channel_list[nbchaines].channel[0]=0;
             }
                                                                                                }
           strcpy(tv_channel_list[nbchaines].name,val);
        continue;
  }

  if (strcmp(tag,"tv_channel")==0) {
                                 strcpy(tv_channel_list[nbchaines].channel,val);
                                 continue;
                                }

  if  (strcmp(tag,"tv_freq")==0){
        tv_channel_list[nbchaines].freq= atoi (val);
         continue;
  }

  if  (strcmp(tag,"tv_norm")==0){
        i=1;
        if (strcmp(val,"secam")==0) { i=0; tv_channel_list[nbchaines].norm = VIDEO_MODE_SECAM; }
        if (strcmp(val,"pal")==0)   { i=0; tv_channel_list[nbchaines].norm = VIDEO_MODE_PAL;   }
        if (strcmp(val,"ntsc")==0)  { i=0; tv_channel_list[nbchaines].norm = VIDEO_MODE_NTSC;  }
        if (strcmp(val,"auto")==0)  { i=0; tv_channel_list[nbchaines].norm = VIDEO_MODE_AUTO;  }
	if (i && debug) fprintf(stderr,"%s:line %d: wrong value for %s\n",filename,nl,tag);
	 continue;
   }

  if (debug) fprintf(stderr,"%s:line %d: unknown variable  %s\n",filename,nl,tag);

 } /* end while parsing */

  /* Ranger le dernier canal si il y a tv_freq ou tv_channel */
      if ((tv_channel_list[nbchaines].freq!=0) || ( tv_channel_list[nbchaines].channel[0]!=0)){
      if (nbchaines>=(NB_TV_CHANNEL_MAX - 1)){
      if (debug)
      fprintf(stderr,"Channel limit to %d :line %d  %s : file %s\n",NB_TV_CHANNEL_MAX,nl,tag,filename);
                                              }
      else nbchaines++;
                                                                                              }

 fclose (fp);
 fp = NULL;    	
}


